@@ -1,6 +1,6 @@
 /*
  * Hedgewars, a free turn based strategy game
- * Copyright (c) 2004-2012 Andrey Korotaev <unC0Rr@gmail.com>
+ * Copyright (c) 2004-2013 Andrey Korotaev <unC0Rr@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,9 @@
  * @brief ThemeModel class implementation
  */
 
+#include "physfs.h"
 #include "ThemeModel.h"
+#include "hwconsts.h"
 
 ThemeModel::ThemeModel(QObject *parent) :
     QAbstractListModel(parent)
@@ -51,7 +53,6 @@ void ThemeModel::loadThemes()
 {
     beginResetModel();
 
-
     DataManager & datamgr = DataManager::instance();
 
     QStringList themes =
@@ -66,24 +67,30 @@ void ThemeModel::loadThemes()
     foreach (QString theme, themes)
     {
         // themes without icon are supposed to be hidden
-        QString iconpath =
-            datamgr.findFileForRead(QString("Themes/%1/icon.png").arg(theme));
+        QString iconpath = QString("physfs://Themes/%1/icon.png").arg(theme);
 
         if (!QFile::exists(iconpath))
             continue;
 
         QMap<int, QVariant> dataset;
 
-        // set name
-        dataset.insert(Qt::DisplayRole, theme);
+        // detect if theme is dlc
+        QString themeDir = PHYSFS_getRealDir(QString("Themes/%1/icon.png").arg(theme).toLocal8Bit().data());
+        bool isDLC = !themeDir.startsWith(datadir->absolutePath());
+        dataset.insert(IsDlcRole, isDLC);
 
-        // load and set icon
-        QIcon icon(iconpath);
-        dataset.insert(Qt::DecorationRole, icon);
+        // set icon path
+        dataset.insert(IconPathRole, iconpath);
+
+        // set name
+        dataset.insert(ActualNameRole, theme);
+
+        // set displayed name
+        dataset.insert(Qt::DisplayRole, (isDLC ? "*" : "") + theme);
 
         // load and set preview icon
-        QIcon preview(datamgr.findFileForRead(QString("Themes/%1/icon@2x.png").arg(theme)));
-        dataset.insert(Qt::UserRole, preview);
+        QIcon preview(QString("physfs://Themes/%1/icon@2x.png").arg(theme));
+        dataset.insert(Qt::DecorationRole, preview);
 
         m_data.append(dataset);
     }
@@ -91,7 +98,3 @@ void ThemeModel::loadThemes()
 
     endResetModel();
 }
-
-
-
-
