@@ -1,6 +1,6 @@
 /*
  * Hedgewars, a free turn based strategy game
- * Copyright (c) 2004-2012 Andrey Korotaev <unC0Rr@gmail.com>
+ * Copyright (c) 2004-2013 Andrey Korotaev <unC0Rr@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include <QLabel>
 #include <QSize>
 #include <QFontMetricsF>
+#include <QDebug>
 
 #include "qpushbuttonwithsound.h"
 
@@ -39,27 +40,36 @@ AbstractPage::AbstractPage(QWidget* parent)
 void AbstractPage::initPage()
 {
     QGridLayout * pageLayout = new QGridLayout(this);
+    QHBoxLayout * bottomLeftLayout = new QHBoxLayout();
+    pageLayout->addLayout(bottomLeftLayout, 1, 0);
 
     // stretch grid space for body and footer
-    pageLayout->setColumnStretch(0,0);
-    pageLayout->setColumnStretch(1,1);
+    pageLayout->setColumnStretch(0,1);
+    pageLayout->setColumnStretch(1,2);
+    pageLayout->setColumnStretch(2,1);
     pageLayout->setRowStretch(0,1);
     pageLayout->setRowStretch(1,0);
 
     // add back/exit button
     btnBack = formattedButton(":/res/Exit.png", true);
     btnBack->setWhatsThis(tr("Go back"));
-    pageLayout->addWidget(btnBack, 1, 0, 1, 1, Qt::AlignLeft | Qt::AlignBottom);
+    bottomLeftLayout->addWidget(btnBack, 0, Qt::AlignBottom);
 
     // add body layout as defined by the subclass
     pageLayout->addLayout(bodyLayoutDefinition(), 0, 0, 1, 3);
 
+    // add left footer layout
+    QLayout * flld = footerLayoutLeftDefinition();
+    if (flld != NULL)
+        bottomLeftLayout->addLayout(flld, 0);
+
     descLabel = new QLabel();
     descLabel->setAlignment(Qt::AlignCenter);
     descLabel->setWordWrap(true);
     descLabel->setOpenExternalLinks(true);
     descLabel->setFixedHeight(50);
     descLabel->setStyleSheet("font-size: 16px");
+    bottomLeftLayout->addWidget(descLabel);
     pageLayout->addWidget(descLabel, 1, 1);
 
     // add footer layout
@@ -67,6 +77,8 @@ void AbstractPage::initPage()
     if (fld != NULL)
         pageLayout->addLayout(fld, 1, 2);
 
+    bottomLeftLayout->addStretch(1);
+
     // connect signals
     connect(btnBack, SIGNAL(clicked()), this, SIGNAL(goBack()));
     connectSignals();
@@ -146,13 +158,23 @@ void AbstractPage::setButtonDescription(QString desc)
     descLabel->setText(desc);
 }
 
-void AbstractPage::setDefautDescription(QString text)
+void AbstractPage::setDefaultDescription(QString text)
 {
     *defautDesc = text;
     descLabel->setText(text);
 }
 
-QString * AbstractPage::getDefautDescription()
+QString * AbstractPage::getDefaultDescription()
 {
     return defautDesc;
 }
+
+void AbstractPage::triggerPageEnter()
+{
+    emit pageEnter();
+}
+
+void AbstractPage::triggerPageLeave()
+{
+    emit pageLeave();
+}
