@@ -1,6 +1,6 @@
 /*
  * Hedgewars, a free turn based strategy game
- * Copyright (c) 2004-2012 Andrey Korotaev <unC0Rr@gmail.com>
+ * Copyright (c) 2004-2013 Andrey Korotaev <unC0Rr@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,15 +21,20 @@
 
 #include "AbstractPage.h"
 
+class QIcon;
+
 class PageMain : public AbstractPage
 {
-        Q_OBJECT
+    Q_OBJECT
 
     public:
         PageMain(QWidget * parent = 0);
+        void resetNetworkChoice();
 
         QPushButton * BtnSinglePlayer;
         QPushButton * BtnNet;
+        QPushButton * BtnNetLocal;
+        QPushButton * BtnNetOfficial;
         QPushButton * BtnSetup;
         QPushButton * BtnFeedback;
         QPushButton * BtnInfo;
@@ -41,8 +46,12 @@ class PageMain : public AbstractPage
         QLayout * bodyLayoutDefinition();
         QLayout * footerLayoutDefinition();
         void connectSignals();
+        QIcon originalNetworkIcon, disabledNetworkIcon;
 
         QString randomTip() const;
+
+    private slots:
+        void toggleNetworkChoice();
 };
 
 #endif
