@@ -2,7 +2,6 @@
 module HWProtoInRoomState where
 
 import qualified Data.Map as Map
-import Data.Sequence((|>))
 import Data.List as L
 import Data.Maybe
 import qualified Data.ByteString.Char8 as B
@@ -28,7 +27,7 @@ handleCmd_inRoom ["PART", msg] = return [MoveToLobby $ "part: " `B.append` msg]
 
 
 handleCmd_inRoom ("CFG" : paramName : paramStrs)
-    | null paramStrs = return [ProtocolError "Empty config entry"]
+    | null paramStrs = return [ProtocolError $ loc "Empty config entry"]
     | otherwise = do
         chans <- roomOthersChans
         cl <- thisClient
@@ -37,7 +36,7 @@ handleCmd_inRoom ("CFG" : paramName : paramStrs)
                 ModifyRoom f,
                 AnswerClients chans ("CFG" : paramName : paramStrs)]
             else
-            return [ProtocolError "Not room master"]
+            return [ProtocolError $ loc "Not room master"]
     where
         f r = if paramName `Map.member` (mapParams r) then
                 r{mapParams = Map.insert paramName (head paramStrs) (mapParams r)}
@@ -45,7 +44,7 @@ handleCmd_inRoom ("CFG" : paramName : paramStrs)
                 r{params = Map.insert paramName paramStrs (params r)}
 
 handleCmd_inRoom ("ADD_TEAM" : tName : color : grave : fort : voicepack : flag : difStr : hhsInfo)
-    | length hhsInfo /= 16 = return [ProtocolError "Corrupted hedgehogs info"]
+    | length hhsInfo /= 16 = return [ProtocolError $ loc "Corrupted hedgehogs info"]
     | otherwise = do
         (ci, _) <- ask
         rm <- thisRoom
@@ -55,38 +54,41 @@ handleCmd_inRoom ("ADD_TEAM" : tName : color : grave : fort : voicepack : flag :
         roomChans <- roomClientsChans
         cl <- thisClient
         teamColor <-
-            if clientProto cl < 42 then 
+            if clientProto cl < 42 then
                 return color
                 else
                 liftM (head . (L.\\) (map B.singleton ['0'..]) . map teamcolor . teams) thisRoom
+        let roomTeams = teams rm
+        let hhNum = let p = if not $ null roomTeams then minimum [hhnum $ head roomTeams, canAddNumber roomTeams] else 4 in newTeamHHNum roomTeams p
+        let newTeam = clNick `seq` TeamInfo ci clNick tName teamColor grave fort voicepack flag dif hhNum (hhsList hhsInfo)
         return $
-            if not . null . drop (maxTeams rm - 1) $ teams rm then
-                [Warning "too many teams"]
-            else if canAddNumber rm <= 0 then
-                [Warning "too many hedgehogs"]
+            if not . null . drop (maxTeams rm - 1) $ roomTeams then
+                [Warning $ loc "too many teams"]
+            else if canAddNumber roomTeams <= 0 then
+                [Warning $ loc "too many hedgehogs"]
             else if isJust $ findTeam rm then
-                [Warning "There's already a team with same name in the list"]
+                [Warning $ loc "There's already a team with same name in the list"]
             else if isJust $ gameInfo rm then
-                [Warning "round in progress"]
+                [Warning $ loc "round in progress"]
             else if isRestrictedTeams rm then
-                [Warning "restricted"]
+                [Warning $ loc "restricted"]
             else
-                [ModifyRoom (\r -> r{teams = teams r ++ [newTeam ci clNick r teamColor]}),
+                [ModifyRoom (\r -> r{teams = teams r ++ [newTeam]}),
                 SendUpdateOnThisRoom,
                 ModifyClient (\c -> c{teamsInGame = teamsInGame c + 1, clientClan = Just teamColor}),
                 AnswerClients clChan ["TEAM_ACCEPTED", tName],
-                AnswerClients othChans $ teamToNet $ newTeam ci clNick rm teamColor,
-                AnswerClients roomChans ["TEAM_COLOR", tName, teamColor]
+                AnswerClients othChans $ teamToNet $ newTeam,
+                AnswerClients roomChans ["TEAM_COLOR", tName, teamColor],
+                AnswerClients roomChans ["HH_NUM", tName, showB $ hhnum newTeam]
                 ]
         where
-        canAddNumber r = 48 - (sum . map hhnum $ teams r)
+        canAddNumber rt = (48::Int) - (sum $ map hhnum rt)
         findTeam = find (\t -> tName == teamname t) . teams
-        newTeam ci clNick r tColor = TeamInfo ci clNick tName tColor grave fort voicepack flag dif (newTeamHHNum r) (hhsList hhsInfo)
         dif = readInt_ difStr
         hhsList [] = []
         hhsList [_] = error "Hedgehogs list with odd elements number"
         hhsList (n:h:hhs) = HedgehogInfo n h : hhsList hhs
-        newTeamHHNum r = min 4 (canAddNumber r)
+        newTeamHHNum rt p = min p (canAddNumber rt)
         maxTeams r
             | roomProto r < 38 = 6
             | otherwise = 8
@@ -95,43 +97,44 @@ handleCmd_inRoom ("ADD_TEAM" : tName : color : grave : fort : voicepack : flag :
 handleCmd_inRoom ["REMOVE_TEAM", tName] = do
         (ci, _) <- ask
         r <- thisRoom
-        clNick <- clientNick
 
         let maybeTeam = findTeam r
         let team = fromJust maybeTeam
 
         return $
-            if isNothing $ findTeam r then
-                [Warning "REMOVE_TEAM: no such team"]
-            else if clNick /= teamowner team then
-                [ProtocolError "Not team owner!"]
+            if isNothing $ maybeTeam then
+                [Warning $ loc "REMOVE_TEAM: no such team"]
+            else if ci /= teamownerId team then
+                [ProtocolError $ loc "Not team owner!"]
             else
                 [RemoveTeam tName,
-                SendUpdateOnThisRoom,
                 ModifyClient
                     (\c -> c{
                         teamsInGame = teamsInGame c - 1,
-                        clientClan = if teamsInGame c == 1 then Nothing else Just $ anotherTeamClan ci r
+                        clientClan = if teamsInGame c == 1 then Nothing else Just $ anotherTeamClan ci team r
                     })
                 ]
     where
-        anotherTeamClan ci = teamcolor . fromJust . find (\t -> teamownerId t == ci) . teams
+        anotherTeamClan ci team = teamcolor . fromMaybe (error "CHECKPOINT 011") . find (\t -> (teamownerId t == ci) && (t /= team)) . teams
         findTeam = find (\t -> tName == teamname t) . teams
 
 
 handleCmd_inRoom ["HH_NUM", teamName, numberStr] = do
     cl <- thisClient
-    others <- roomOthersChans
     r <- thisRoom
+    clChan <- thisClientChans
+    others <- roomOthersChans
 
     let maybeTeam = findTeam r
     let team = fromJust maybeTeam
 
     return $
         if not $ isMaster cl then
-            [ProtocolError "Not room master"]
-        else if hhNumber < 1 || hhNumber > 8 || isNothing maybeTeam || hhNumber > canAddNumber r + hhnum team then
+            [ProtocolError $ loc "Not room master"]
+        else if isNothing maybeTeam then
             []
+        else if hhNumber < 1 || hhNumber > 8 || hhNumber > canAddNumber r + hhnum team then
+            [AnswerClients clChan ["HH_NUM", teamName, showB $ hhnum team]]
         else
             [ModifyRoom $ modifyTeam team{hhnum = hhNumber},
             AnswerClients others ["HH_NUM", teamName, showB hhNumber]]
@@ -152,7 +155,7 @@ handleCmd_inRoom ["TEAM_COLOR", teamName, newColor] = do
 
     return $
         if not $ isMaster cl then
-            [ProtocolError "Not room master"]
+            [ProtocolError $ loc "Not room master"]
         else if isNothing maybeTeam then
             []
         else
@@ -166,17 +169,16 @@ handleCmd_inRoom ["TEAM_COLOR", teamName, newColor] = do
 handleCmd_inRoom ["TOGGLE_READY"] = do
     cl <- thisClient
     chans <- roomClientsChans
-    if isMaster cl then
-        return []
-        else
-        return [
-            ModifyRoom (\r -> r{readyPlayers = readyPlayers r + (if isReady cl then -1 else 1)}),
-            ModifyClient (\c -> c{isReady = not $ isReady cl}),
-            AnswerClients chans $ if clientProto cl < 38 then
-                    [if isReady cl then "NOT_READY" else "READY", nick cl]
-                    else
-                    ["CLIENT_FLAGS", if isReady cl then "-r" else "+r", nick cl]
-            ]
+
+    return [
+        ModifyRoom (\r -> r{readyPlayers = readyPlayers r + (if isReady cl then -1 else 1)}),
+        ModifyClient (\c -> c{isReady = not $ isReady cl}),
+        AnswerClients chans $ if clientProto cl < 38 then
+                [if isReady cl then "NOT_READY" else "READY", nick cl]
+                else
+                ["CLIENT_FLAGS", if isReady cl then "-r" else "+r", nick cl]
+        ]
+
 
 handleCmd_inRoom ["START_GAME"] = do
     (ci, rnc) <- ask
@@ -187,7 +189,7 @@ handleCmd_inRoom ["START_GAME"] = do
     let nicks = map (nick . client rnc) . roomClients rnc $ clientRoom rnc ci
     let allPlayersRegistered = all ((<) 0 . B.length . webPassword . client rnc . teamownerId) $ teams rm
 
-    if isMaster cl && playersIn rm == readyPlayers rm && not (isJust $ gameInfo rm) then
+    if isMaster cl && (playersIn rm == readyPlayers rm || clientProto cl > 43) && not (isJust $ gameInfo rm) then
         if enoughClans rm then
             return [
                 ModifyRoom
@@ -201,7 +203,7 @@ handleCmd_inRoom ["START_GAME"] = do
                 , ModifyRoomClients (\c -> c{isInGame = True})
                 ]
             else
-            return [Warning "Less than two clans!"]
+            return [Warning $ loc "Less than two clans!"]
         else
         return []
     where
@@ -213,15 +215,16 @@ handleCmd_inRoom ["EM", msg] = do
     rm <- thisRoom
     chans <- roomOthersChans
 
-    if teamsInGame cl > 0 && (isJust $ gameInfo rm) && isLegal then
-        return $ AnswerClients chans ["EM", msg] : [ModifyRoom (\r -> r{gameInfo = liftM (\g -> g{roundMsgs = roundMsgs g |> msg}) $ gameInfo r}) | not isKeepAlive]
+    if teamsInGame cl > 0 && (isJust $ gameInfo rm) && (not $ B.null legalMsgs) then
+        return $ AnswerClients chans ["EM", legalMsgs]
+            : [ModifyRoom (\r -> r{gameInfo = liftM (\g -> g{roundMsgs = nonEmptyMsgs : roundMsgs g}) $ gameInfo r}) | not $ B.null nonEmptyMsgs]
         else
         return []
     where
-        (isLegal, isKeepAlive) = checkNetCmd msg
+        (legalMsgs, nonEmptyMsgs) = checkNetCmd msg
 
 
-handleCmd_inRoom ["ROUNDFINISHED", correctly] = do
+handleCmd_inRoom ["ROUNDFINISHED", _] = do
     cl <- thisClient
     rm <- thisRoom
     chans <- roomClientsChans
@@ -231,16 +234,13 @@ handleCmd_inRoom ["ROUNDFINISHED", correctly] = do
 
     if isInGame cl then
         if isJust $ gameInfo rm then
-            if (isMaster cl && isCorrect) then
-                return $ FinishGame : unsetInGameState
-                else
-                return $ unsetInGameState ++ map SendTeamRemovalMessage clTeams
+            return $ unsetInGameState ++ map SendTeamRemovalMessage clTeams
             else
             return unsetInGameState
         else
         return [] -- don't accept this message twice
     where
-        isCorrect = correctly == "1"
+--        isCorrect = correctly == "1"
 
 -- compatibility with clients with protocol < 38
 handleCmd_inRoom ["ROUNDFINISHED"] =
@@ -250,7 +250,7 @@ handleCmd_inRoom ["TOGGLE_RESTRICT_JOINS"] = do
     cl <- thisClient
     return $
         if not $ isMaster cl then
-            [ProtocolError "Not room master"]
+            [ProtocolError $ loc "Not room master"]
         else
             [ModifyRoom (\r -> r{isRestrictedJoins = not $ isRestrictedJoins r})]
 
@@ -259,11 +259,20 @@ handleCmd_inRoom ["TOGGLE_RESTRICT_TEAMS"] = do
     cl <- thisClient
     return $
         if not $ isMaster cl then
-            [ProtocolError "Not room master"]
+            [ProtocolError $ loc "Not room master"]
         else
             [ModifyRoom (\r -> r{isRestrictedTeams = not $ isRestrictedTeams r})]
 
 
+handleCmd_inRoom ["TOGGLE_REGISTERED_ONLY"] = do
+    cl <- thisClient
+    return $
+        if not $ isMaster cl then
+            [ProtocolError $ loc "Not room master"]
+        else
+            [ModifyRoom (\r -> r{isRegisteredOnly = not $ isRegisteredOnly r})]
+
+
 handleCmd_inRoom ["ROOM_NAME", newName] = do
     cl <- thisClient
     rs <- allRoomInfos
@@ -272,10 +281,10 @@ handleCmd_inRoom ["ROOM_NAME", newName] = do
 
     return $
         if not $ isMaster cl then
-            [ProtocolError "Not room master"]
+            [ProtocolError $ loc "Not room master"]
         else
         if isJust $ find (\r -> newName == name r) rs then
-            [Warning "Room with such name already exists"]
+            [Warning $ loc "Room with such name already exists"]
         else
             [ModifyRoom roomUpdate,
             AnswerClients chans ("ROOM" : "UPD" : name rm : roomInfo (nick cl) (roomUpdate rm))]
@@ -287,10 +296,34 @@ handleCmd_inRoom ["KICK", kickNick] = do
     (thisClientId, rnc) <- ask
     maybeClientId <- clientByNick kickNick
     master <- liftM isMaster thisClient
+    rm <- thisRoom
     let kickId = fromJust maybeClientId
+    let kickCl = rnc `client` kickId
     let sameRoom = clientRoom rnc thisClientId == clientRoom rnc kickId
+    let notOnly2Players = (length . group . sort . map teamowner . teams $ rm) > 2
     return
-        [KickRoomClient kickId | master && isJust maybeClientId && (kickId /= thisClientId) && sameRoom]
+        [KickRoomClient kickId |
+            master
+            && isJust maybeClientId
+            && (kickId /= thisClientId)
+            && sameRoom
+            && ((isNothing $ gameInfo rm) || notOnly2Players || teamsInGame kickCl == 0)
+        ]
+
+
+handleCmd_inRoom ["DELEGATE", newAdmin] = do
+    (thisClientId, rnc) <- ask
+    maybeClientId <- clientByNick newAdmin
+    master <- liftM isMaster thisClient
+    serverAdmin <- liftM isAdministrator thisClient
+    let newAdminId = fromJust maybeClientId
+    let sameRoom = clientRoom rnc thisClientId == clientRoom rnc newAdminId
+    return
+        [ChangeMaster (Just newAdminId) |
+            (master || serverAdmin)
+                && isJust maybeClientId
+                && ((newAdminId /= thisClientId) || (serverAdmin && not master))
+                && sameRoom]
 
 
 handleCmd_inRoom ["TEAMCHAT", msg] = do
@@ -298,15 +331,27 @@ handleCmd_inRoom ["TEAMCHAT", msg] = do
     chans <- roomSameClanChans
     return [AnswerClients chans ["EM", engineMsg cl]]
     where
-        engineMsg cl = toEngineMsg $ B.concat ["b", nick cl, "(team): ", msg, "\x20\x20"]
+        engineMsg cl = toEngineMsg $ B.concat ["b", nick cl, " (team): ", msg, "\x20\x20"]
+
 
 handleCmd_inRoom ["BAN", banNick] = do
-    (_, rnc) <- ask
+    (thisClientId, rnc) <- ask
     maybeClientId <- clientByNick banNick
-    let banId = fromJust maybeClientId
     master <- liftM isMaster thisClient
-    return [ModifyRoom (\r -> r{roomBansList = (host $ rnc `client` banId) : roomBansList r}) | master && isJust maybeClientId]
+    let banId = fromJust maybeClientId
+    let sameRoom = clientRoom rnc thisClientId == clientRoom rnc banId
+    if master && isJust maybeClientId && (banId /= thisClientId) && sameRoom then
+        return [
+--                ModifyRoom (\r -> r{roomBansList = let h = host $ rnc `client` banId in h `deepseq` h : roomBansList r})
+                KickRoomClient banId
+            ]
+        else
+        return []
 
+handleCmd_inRoom ("RND":rs) = do
+    n <- clientNick
+    s <- roomClientsChans
+    return [AnswerClients s ["CHAT", n, B.unwords $ "/rnd" : rs], Random s rs]
 
 handleCmd_inRoom ["LIST"] = return [] -- for old clients (<= 0.9.17)
 
